/*
 * This file is a part of SCP Timer, program for usability experiments
 * Copyright (C) 2007 Martin Plicka
 *
 * SCP Timer is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * SCP Timer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with SCP Timer. If not, see <http://www.gnu.org/licenses/>.
 */

#include "mainwindow.h"

#include <QMessageBox>
#include <QFile>
#include <QFileDialog>
#include <QTextStream>
#include <QFileInfo>
#include <QDesktopWidget>
#include <QTime>


MainWindow::MainWindow (){
   QWidget(NULL, 0);
   setupUi(this);
   step = 1;
   markStep();
   connect (buttonNext, SIGNAL(clicked()), this, SLOT(buttonNextClicked()));
   connect (buttonPrev, SIGNAL(clicked()), this, SLOT(buttonPrevClicked()));
   connect (buttonClose, SIGNAL(clicked()), this, SLOT(buttonCloseClicked()));
   taskWindow = new TaskWindow(qApp->desktop()->availableGeometry(),this);
   overlayWindow = new OverlayWindow(qApp->desktop()->availableGeometry(),this);
}

void MainWindow::closeEvent ( QCloseEvent * event ){
   if (QMessageBox::question ( this, "Opravdu?",
                               "Chcete opravdu ukončit testování?",
                               QMessageBox::Yes| QMessageBox::No,
                               QMessageBox::No) == QMessageBox::Yes )
      qApp->quit();
   else event->ignore();
}

bool MainWindow::loadTask() {
   //smaz text predchoziho testu
   task.clear();

   QString filename = QFileDialog::getOpenFileName(this,
                      tr("Otevrit ukol"), "", tr("Soubory ukolu (*.sce)"));
   if (filename.isNull()) return false;

   QFile file(filename);
   if (!file.open(QIODevice::ReadOnly | QIODevice::Text)){
      QMessageBox::critical(this, "Chyba", "Nepodařilo se otevřít daný soubor");
      return false;
   }

   //zobraz jmeno souboru
   QFileInfo f(filename);
   labelTaskFilename -> setText (f.completeBaseName ());
   labelTaskFilename -> setToolTip (filename);
   QTextStream in(&file);

   while (!in.atEnd()) {
      task.append(in.readLine());
   }
   return true;
}


bool MainWindow::doTask(){
   report.clear();
   QString tmp = "#Velikost pouzitelne plochy: ";
   tmp += QString::number(qApp->desktop()->availableGeometry().width());
   tmp += "x";
   tmp += QString::number(qApp->desktop()->availableGeometry().height());
   report.append(tmp);
   report.append("#");
   for (int i=0; i<task.size(); i++) {
      QString line = task[i];

      //preskoc komentare a prazdne radky
      if (line.trimmed().isEmpty() || line[0] == '#') {
         report.append(line);
         continue;
      }
      report.append(QString("!")+line);

      overlayWindow->labelText->setText(line);
      taskWindow->labelText->setText(line);

      hide();
      overlayWindow->exec();
      QTime start = QTime::currentTime();

      if (!taskWindow->exec()) {
         show();
         return false;
      }
      QTime stop = QTime::currentTime();
      show();
      report.append(QString(">") + start.toString("hh:mm:ss.zzz"));
      report.append(QString("<") + stop.toString("hh:mm:ss.zzz"));
      report.append(QString("=") + QString::number((start.msecsTo(stop))));
   }
   return true;
}


bool MainWindow::saveReport(){
   QString filename = QFileDialog::getSaveFileName(this, "Uložit soubor",
                      "",
                      "Zpráva o průběhu experimentu (*.scp)");
   if (filename.isNull()) return false;

   QFile data(filename);
   if (data.open(QFile::WriteOnly | QFile::Truncate)) {
      QTextStream out(&data);
      out << report.join("\n") << endl;
      out.flush();
      data.close();
   }
   else {
      QMessageBox::critical(this,"Chyba", "Nepovedlo se uložit soubor.\nPravděpodobně nemáte do zvoleného adresáře právo zapisovat.");
      return false;
   }
   labelSavedFile->setText(filename);
   return true;
}
void MainWindow::evalStep(){
   switch (step) {
   case 2: //prechod na otevreni testu
      break;
   case 3: //otevirani testu
      if (!loadTask()) step--;
      break;
   case 4: //testovani

      if (!doTask()) step--;
      break;
   case 5: //ukladani testu
      if (!saveReport()) step--;
      break;
   case 1:
   default:
      step = 1;
      break;
   }
}

void MainWindow::markStep(){
   if (step<1 || step> 5) step=0;
   stackedWidget->setCurrentIndex(step-1);

   //set form for labels
   QFont f[2];
   f[1].setBold(true);

   //set bold if current step is given number
   labelStep1->setFont(f[step==1]);
   labelStep2->setFont(f[step==2]);
   labelStep3->setFont(f[step==3]);
   labelStep4->setFont(f[step==4]);
   labelStep5->setFont(f[step==5]);

   switch (step){
   case 1:
      buttonPrev->setEnabled(false);
      buttonNext->setText("Další >>");
      buttonClose->setEnabled(true);
      frameTaskFilename->hide();
      break;
   case 2:
      buttonPrev->setEnabled(true);
      buttonClose->setEnabled(true);
      frameTaskFilename->hide();
      break;
   case 3:
      buttonPrev->setEnabled(true);
      buttonClose->setEnabled(true);
      frameTaskFilename->show();
      break;
   case 4:
      buttonPrev->setEnabled(false);
      buttonClose->setEnabled(false);
      break;
   case 5:
      buttonPrev->setEnabled(false);
      buttonNext->setText("Další test");
      buttonClose->setEnabled(true);
      break;
   }
}

void MainWindow::buttonNextClicked(){
   step++;
   evalStep();
   markStep();
}

void MainWindow::buttonPrevClicked(){
   step--;
   evalStep();
   markStep();
}

void MainWindow::buttonCloseClicked(){
   close();
}
